Function LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata = videoMetadata.Lookup(videoPlayerKey)
    if metadata = Invalid then
        return CreateObject("roAssociativeArray")
    endif
    return metadata
End Function

Sub SetMetadataType(videoMetadata, videoPlayerKey, metadataType)
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata["type"] = metadataType
    videoMetadata.AddReplace(videoPlayerKey, metadata)
End Sub

Sub SetVideoMetadata(videoPlayers, videoMetadata, videoPlayerKey)
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    if metadata["type"] = "video" then
        videoPlayer = videoPlayers.Lookup(videoPlayerKey)
        metadata["timer"] = CreateObject("roTimeSpan")
        metadata["duration"] = videoPlayer.GetDuration()
        metadata["stopped"] = false
        videoMetadata.AddReplace(videoPlayerKey, metadata)
    endif
End Sub

Sub SetMetadataStopped(videoMetadata, videoPlayerKey)
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata["stopped"] = true
    videoMetadata.AddReplace(videoPlayerKey, metadata)
End Sub

Sub AppendEndedEvent(videoPlayer)
    videoDuration = videoPlayer.GetDuration()
    videoPlayer.AddEvent(8, videoDuration - 10)
End Sub

Sub PrepareMedia(screenRegistrySection, videoModeProvider, videoPlayer, event)          
    x = strtoi(event.message.x)
    y = strtoi(event.message.y)
    width = strtoi(event.message.width)
    height = strtoi(event.message.height)
    orientation = GetCurrentVideoTransform(screenRegistrySection)
    rectangle = CreateObject("roRectangle", x, y, width, height)
    videoPlayer.SetRectangle(rectangle)
    videoPlayer.SetTransform(orientation)

    ' Loop mode needs to be set before video prepare otherwise loop mode won't work even
    ' if "GetPropertiesString" method will return right value
    videoPlayer.SetLoopMode("AlwaysLoop")     

    if event.message.background = "true" then
        videoModeProvider.SetGraphicsZOrder("front")
    else 
        videoModeProvider.SetGraphicsZOrder("back")
    endif
End Sub

Sub PrepareVideo(videoPlayer, filePath, volume)
    videoPlayer.SetVolume(volume)
    videoPlayer.PreloadFile({ filename: filePath })
End Sub

Sub PlayVideo(videoPlayer, filePath, volume)
    videoPlayer.SetVolume(volume)
    videoPlayer.PlayFile(filePath)
End Sub

Sub PrepareStream(videoPlayer, uri)
    stream = CreateObject("roRtspStream", uri)

    print "PreloadFile:Stream"

    videoPlayer.PreloadFile({ rtsp: stream })
End Sub

Sub PlayStream(videoPlayer, uri)
    stream = CreateObject("roRtspStream", uri)
    videoPlayer.PlayFile({ rtsp: stream })
End Sub

Sub PlayHDMI(videoPlayer)
    videoInput = CreateObject("roVideoInput")
    videoPlayer.PlayFile(videoInput)
End Sub

Function GetSupportedVideoModes(videoModeProvider)
    allModes = videoModeProvider.GetAvailableModes()
    supportedModes = CreateObject("roArray", 0, true)
    includedModes = CreateObject("roAssociativeArray")
    While allModes.IsNext()
        mode = allModes.Next()
        modeKey = CreateObject("roString")
        width = mode["width"].ToStr()
        height = mode["height"].ToStr()
        framerate = mode["framerate"].ToStr()
        modeKey.AppendString(width, width.Len())
        modeKey.AppendString("x", 1)
        modeKey.AppendString(height, height.Len())
        modeKey.AppendString("@", 1)
        modeKey.AppendString(framerate, framerate.Len())
        ' Videos in PORTRAIT orientation can not be played in interlaced modes
        if mode.interlaced = false and not(includedModes.DoesExist(modeKey)) then
            includedModes.AddReplace(modeKey, modeKey)
            supportedModes.Push(mode)
        endif
    End While
    return supportedModes
End Function

Sub InitVideoMode(screenRegistrySection, supportedVideoModes, defaultWidth, defaultHeight)
    if not(screenRegistrySection.Exists("video_mode")) then
        SaveVideoModeIfSupported(screenRegistrySection, supportedVideoModes, defaultWidth, defaultHeight, "auto")
    endif
End Sub

Function SaveVideoModeIfSupported(screenRegistrySection, supportedVideoModes, width, height, framerate)
    supportedVideoModes.Reset()
    While supportedVideoModes.IsNext()
        mode = supportedVideoModes.Next()
        if (framerate = "auto" OR mode["framerate"] = strtoi(framerate)) AND mode["width"] = strtoi(width) AND mode["height"] = strtoi(height) then
            screenRegistrySection.Write("video_mode", FormatJson(mode, 1))
            screenRegistrySection.Flush()
            return true
        endif
    End While
    return false
End Function

Sub ApplySavedVideoMode(screenRegistrySection, videoModeProvider, fullScreenRectangle, touchScreen)
    if screenRegistrySection.Exists("video_mode") then
        videoMode = ParseJson(screenRegistrySection.Read("video_mode"))
        width = videoMode["graphicsPlaneWidth"]
        height = videoMode["graphicsPlaneHeight"]
        fullScreenRectangle.SetWidth(width)
        fullScreenRectangle.SetHeight(height)
        touchScreen.SetResolution(width, height)
        videoModeProvider.SetMode(videoMode["videomode"])
    endif
End Sub

Function GetCurrentVideoTransform(screenRegistrySection)
    if screenRegistrySection.Exists("video_orientation") then
        return "identity" ' LANDSCAPE_FLIPPED is not supported on this platform
    endif
    return GetCurrentTransform(screenRegistrySection)
End Function

Function GetCurrentTransform(screenRegistrySection)
    orientation = screenRegistrySection.Read("orientation")
    if orientation = "LANDSCAPE" then
        return "identity"
    endif
    if orientation = "PORTRAIT" then
        return "rot90"
    endif
    if orientation = "PORTRAIT_FLIPPED" then
        return "rot270"
    endif
    return "identity"
End Function
